################################################################################
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
# limitations under the License.
#################################################################################
from typing import List
from uuid import UUID

from flink_agents.api.agents.types import OutputSchema
from flink_agents.api.chat_message import ChatMessage
from flink_agents.api.events.event import Event


class ChatRequestEvent(Event):
    """Event representing a request to chat model.

    Attributes:
    ----------
    model : str
        The name of the chat model to be chatted with.
    messages : List[ChatMessage]
        The input to the chat model.
    output_schema: OutputSchema | None
        The expected output schema of the chat model final response. Optional.
    """

    model: str
    messages: List[ChatMessage]
    output_schema: OutputSchema | None = None


class ChatResponseEvent(Event):
    """Event representing a response from chat model.

    Attributes:
    ----------
    request_id : UUID
        The id of the request event.
    response : ChatMessage
        The response from the chat model.
    """

    request_id: UUID
    response: ChatMessage



################################################################################
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
# limitations under the License.
#################################################################################
from typing import Any, Type

import pytest
from pydantic import ValidationError
from pydantic_core import PydanticSerializationError
from pyflink.common import Row

from flink_agents.api.events.event import Event, InputEvent, OutputEvent


def test_event_init_serializable() -> None:  # noqa D103
    Event(a=1, b=InputEvent(input=1), c=OutputEvent(output="111"))


def test_event_init_non_serializable() -> None:  # noqa D103
    with pytest.raises(ValidationError):
        Event(a=1, b=Type[InputEvent])


def test_event_setattr_serializable() -> None:  # noqa D103
    event = Event(a=1)
    event.c = Event()


def test_event_setattr_non_serializable() -> None:  # noqa D103
    event = Event(a=1)
    with pytest.raises(PydanticSerializationError):
        event.c = Type[InputEvent]


def test_input_event_ignore_row_unserializable() -> None:  # noqa D103
    InputEvent(input=Row({"a": 1}))


def test_event_row_with_non_serializable_fails() -> None:  # noqa D103
    with pytest.raises(ValidationError):
        Event(row_field=Row({"a": 1}), non_serializable_field=Type[InputEvent])


def test_event_multiple_rows_serializable() -> None:  # noqa D103
    Event(row1=Row({"a": 1}), row2=Row({"b": 2}), normal_field="test")


def test_event_setattr_row_serializable() -> None:  # noqa D103
    event = Event(a=1)
    event.row_field = Row({"key": "value"})


def test_event_json_serialization_with_row() -> None:  # noqa D103
    event = InputEvent(input=Row({"test": "data"}))
    json_str = event.model_dump_json()
    assert "test" in json_str
    assert "Row" in json_str


def test_efficient_row_serialization_with_fallback() -> None:
    """Test that the new fallback-based serialization works efficiently."""
    row_data = {"a": 1, "b": "test", "c": [1, 2, 3]}
    event = InputEvent(input=Row(row_data))

    json_str = event.model_dump_json()
    import json

    parsed = json.loads(json_str)

    assert parsed["input"]["type"] == "Row"
    assert parsed["input"]["values"] == [row_data]
    assert "id" in parsed  # UUID should be present

    def custom_fallback(obj: Any) -> dict[str, Any]:
        if isinstance(obj, Row):
            return {"custom_type": "CustomRow", "data": obj._values}
        msg = "Unknown type"
        raise ValueError(msg)

    custom_json = event.model_dump_json(fallback=custom_fallback)
    custom_parsed = json.loads(custom_json)

    assert custom_parsed["input"]["custom_type"] == "CustomRow"
    assert custom_parsed["input"]["data"] == [row_data]


def test_event_with_mixed_serializable_types() -> None:
    """Test event with mix of normal and Row types."""
    event = InputEvent(
        input={
            "normal_data": {"key": "value"},
            "row_data": Row({"test": "data"}),
            "list_data": [1, 2, 3],
            "nested_row": {"inner": Row({"nested": True})},
        }
    )

    json_str = event.model_dump_json()

    import json

    parsed = json.loads(json_str)

    # Normal data should be serialized normally
    assert parsed["input"]["normal_data"]["key"] == "value"
    assert parsed["input"]["list_data"] == [1, 2, 3]

    # Row data should use fallback serializer
    assert parsed["input"]["row_data"]["type"] == "Row"
    assert parsed["input"]["nested_row"]["inner"]["type"] == "Row"


def test_input_event_ignore_row_unserializable() -> None:  # noqa D103
    InputEvent(input=Row({"a": 1}))



################################################################################
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
# limitations under the License.
#################################################################################
import json
import os
import shutil
import sysconfig
import tempfile
from pathlib import Path

from pyflink.common import Configuration, WatermarkStrategy
from pyflink.datastream import (
    KeySelector,
    RuntimeExecutionMode,
    StreamExecutionEnvironment,
)
from pyflink.datastream.connectors.file_system import (
    FileSource,
    StreamFormat,
)

from flink_agents.api.agents.agent import Agent
from flink_agents.api.decorators import action
from flink_agents.api.events.event import InputEvent, OutputEvent
from flink_agents.api.execution_environment import AgentsExecutionEnvironment
from flink_agents.api.runner_context import RunnerContext

os.environ["PYTHONPATH"] = sysconfig.get_paths()["purelib"]


class InputKeySelector(KeySelector):
    """Key selector for input data."""

    def get_key(self, value: dict) -> int:
        """Extract key from input data."""
        return value.get("id", 0)


class PythonEventLoggingAgent(Agent):
    """Agent for testing PythonEvent logging."""

    @action(InputEvent)
    @staticmethod
    def process_input(event: InputEvent, ctx: RunnerContext) -> None:
        """Process input event and send a PythonEvent."""
        # Send a PythonEvent that should be logged with readable content
        input_data = event.input
        ctx.send_event(
            OutputEvent(output={"processed_review": f"{input_data['review']}"})
        )


def test_python_event_logging(tmp_path: Path) -> None:
    """Test event logs are written to configured directory with expected content."""
    event_log_dir = tmp_path / "event_log"
    default_log_dir = Path(tempfile.gettempdir()) / "flink-agents"
    shutil.rmtree(default_log_dir, ignore_errors=True)

    config = Configuration()
    env = StreamExecutionEnvironment.get_execution_environment(config)
    env.set_runtime_mode(RuntimeExecutionMode.STREAMING)
    env.set_parallelism(1)

    # Create agent environment
    agents_env = AgentsExecutionEnvironment.get_execution_environment(env=env)
    agents_env.get_config().set_str("baseLogDir", str(event_log_dir))

    # Set up input source
    current_dir = Path(__file__).parent
    input_datastream = env.from_source(
        source=FileSource.for_record_stream_format(
            StreamFormat.text_line_format(),
            f"file:///{current_dir}/../resources/input/input_data.txt",
        ).build(),
        watermark_strategy=WatermarkStrategy.no_watermarks(),
        source_name="python_event_logging_test",
    )

    # Parse input
    deserialize_datastream = input_datastream.map(lambda x: json.loads(x))

    # Apply agent
    agents_env.from_datastream(
        input=deserialize_datastream, key_selector=InputKeySelector()
    ).apply(PythonEventLoggingAgent()).to_datastream()

    # Execute the job
    agents_env.execute()

    # Check that log files were created in configured directory
    log_files = list(event_log_dir.glob("events-*.log"))

    # At least one log file should exist
    assert len(log_files) > 0, (
        f"Event log files should be created in {event_log_dir}"
    )

    # Check that log files contain structured event content
    record = None
    record_line = None
    has_processed_review = False
    for log_file in log_files:
        with log_file.open(encoding="utf-8") as handle:
            for line in handle:
                if line.strip():
                    record = json.loads(line)
                    record_line = line
                    event_payload = record.get("event", {})
                    if "processed_review" in json.dumps(event_payload):
                        has_processed_review = True
                        break
        if record is not None and has_processed_review:
            break

    assert record is not None, "Event log file is empty."
    assert record_line is not None, "Event log file is empty."
    assert "timestamp" in record
    assert "event" in record
    assert "eventType" in record["event"]
    assert has_processed_review, "Log should contain processed review content"

    event_type_idx = record_line.find('"eventType"')
    id_idx = record_line.find('"id"')
    attributes_idx = record_line.find('"attributes"')
    assert event_type_idx != -1
    assert id_idx != -1
    assert attributes_idx != -1
    assert event_type_idx < id_idx < attributes_idx



################################################################################
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
# limitations under the License.
#################################################################################

try:
    import dotenv
    dotenv.load_dotenv()
except ImportError:
    # dotenv is optional for this test server
    pass

from mcp.server.fastmcp import FastMCP

# Create MCP server
mcp = FastMCP("BasicServer")


@mcp.prompt()
def ask_sum(a: int, b: int) -> str:
    """Prompt of add tool."""
    return f"Can you please calculate the sum of {a} and {b}?"

@mcp.tool()
async def add(a: int, b: int) -> int:
    """Get the detailed information of a specified IP address.

    Args:
        a: The first operand.
        b: The second operand.

    Returns:
        int: The sum of a and b.
    """
    return a + b

mcp.run("streamable-http")




################################################################################
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
# limitations under the License.
#################################################################################
from typing import List
from uuid import UUID

from flink_agents.api.events.event import Event
from flink_agents.api.vector_stores.vector_store import Document


class ContextRetrievalRequestEvent(Event):
    """Event representing a request for context retrieval.

    Attributes:
    ----------
    query : str
        The search query text to find relevant context for
    vector_store : str
        Name of the vector store setup resource to use
    max_results : int
        Maximum number of results to return (default: 3)
    """
    query: str
    vector_store: str
    max_results: int = 3


class ContextRetrievalResponseEvent(Event):
    """Event representing retrieved context results.

    Attributes:
    ----------
    request_id : UUID
        ID of the original request event
    query : str
        The original search query from the request
    documents : List[Document]
        List of retrieved documents from the vector store
    """
    request_id: UUID
    query: str
    documents: List[Document]



################################################################################
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
# limitations under the License.
#################################################################################
from abc import ABC, abstractmethod
from enum import Enum
from typing import TYPE_CHECKING, Any, Dict, List, Union

from pydantic import BaseModel

if TYPE_CHECKING:
    from flink_agents.api.memory_reference import MemoryRef

class MemoryType(Enum):
    """Memory types based on MemoryObject."""
    SENSORY = "sensory",
    SHORT_TERM = "short_term"

class MemoryObject(BaseModel, ABC):
    """Representation of an object in the short-term memory.

    A direct field is a field which stores concrete data directly, while an indirect
    filed is just a field which represents a nested object. Fields can be accessed
    using an absolute or relative path.
    """

    @abstractmethod
    def get(self, path_or_ref: Union[str,"MemoryRef"] ) -> Any:
        """Get the value of a (direct or indirect) field or a MemoryRef in the object.

        Parameters
        ----------
        path_or_ref: Union[str,MemoryRef]
          Relative path from the current object to the target field or
          a MemoryRef instance.

        Returns:
        -------
        Any
          If the input is a MemoryRef, resolve the reference and returns the data.
          If the field is a direct field, return the concrete data stored.
          If the field is an indirect field, another MemoryObject will be returned.
          If the field doesn't exist, return None.
        """

    @abstractmethod
    def set(self, path: str, value: Any) -> "MemoryRef":
        """Set the value of a direct field in the object and return a reference to it.
        This will also create the intermediate objects if not exist.

        Parameters
        ----------
        path: str
          Relative path from the current object to the target field.
        value: Any
          New value of the field. The type of the value must be a primary type.

        Returns:
        -------
        MemoryRef
          A newly created reference to the data just set.
        """

    @abstractmethod
    def new_object(self, path: str) -> "MemoryObject":
        """Create a new object as the value of an indirect field in the object.

        Parameters
        ----------
        path: str
          Relative path from the current object to the target field.

        Returns:
        -------
        MemoryObject
          The created object.
        """

    @abstractmethod
    def is_exist(self, path: str) -> bool:
        """Check whether a (direct or indirect) field exist in the object.

        Parameters
        ----------
        path: str
          Relative path from the current object to the target field.

        Returns:
        -------
        bool
          Whether the field exists.
        """

    @abstractmethod
    def get_field_names(self) -> List[str]:
        """Get names of all the top-level subfields of the object.

        Returns:
        -------
        List[str]
          Top-level subfield names of the object in a list.
        """

    @abstractmethod
    def get_fields(self) -> Dict[str, Any]:
        """Get all the top-level subfields of the object.

        Returns:
        -------
        Dict[str, Any]
          Top-level subfields of the object in a dictionary.
        """



################################################################################
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
# limitations under the License.
#################################################################################
import os

from flink_agents.api.agents.agent import Agent
from flink_agents.api.chat_message import ChatMessage, MessageRole
from flink_agents.api.decorators import (
    action,
    chat_model_connection,
    chat_model_setup,
    prompt,
    tool,
)
from flink_agents.api.events.chat_event import ChatRequestEvent, ChatResponseEvent
from flink_agents.api.events.event import InputEvent, OutputEvent
from flink_agents.api.prompts.prompt import Prompt
from flink_agents.api.resource import (
    ResourceDescriptor,
    ResourceName,
)
from flink_agents.api.runner_context import RunnerContext


class ChatModelCrossLanguageAgent(Agent):
    """Example agent demonstrating cross-language integration testing.

    This test includes:
    - Python scheduling Java ChatModel
    - Java ChatModel scheduling Python Prompt
    - Java ChatModel scheduling Python Tool
    """

    @prompt
    @staticmethod
    def from_text_prompt() -> Prompt:
        """Prompt for instruction."""
        return Prompt.from_text("Please answer the user's question.")

    @prompt
    @staticmethod
    def from_messages_prompt() -> Prompt:
        """Prompt for instruction."""
        return Prompt.from_messages(
            messages=[
                ChatMessage(
                    role=MessageRole.SYSTEM,
                    content="Please answer the user's question.",
                ),
            ],
        )

    @chat_model_connection
    @staticmethod
    def ollama_connection_python() -> ResourceDescriptor:
        """ChatModelConnection responsible for ollama model service connection."""
        return ResourceDescriptor(
            clazz=ResourceName.ChatModel.OLLAMA_CONNECTION, request_timeout=240.0
        )

    @chat_model_connection
    @staticmethod
    def ollama_connection_java() -> ResourceDescriptor:
        """ChatModelConnection responsible for ollama model service connection."""
        return ResourceDescriptor(
            clazz=ResourceName.ChatModel.JAVA_WRAPPER_CONNECTION,
            java_clazz=ResourceName.ChatModel.Java.OLLAMA_CONNECTION,
            endpoint="http://localhost:11434",
            requestTimeout=120,
        )

    @chat_model_setup
    @staticmethod
    def math_chat_model() -> ResourceDescriptor:
        """ChatModel which focus on math, and reuse ChatModelConnection."""
        return ResourceDescriptor(
            clazz=ResourceName.ChatModel.JAVA_WRAPPER_SETUP,
            java_clazz=ResourceName.ChatModel.Java.OLLAMA_SETUP,
            connection="ollama_connection_python",
            model=os.environ.get("OLLAMA_CHAT_MODEL", "qwen3:1.7b"),
            prompt="from_messages_prompt",
            tools=["add"],
            extract_reasoning=True,
        )

    @chat_model_setup
    @staticmethod
    def creative_chat_model() -> ResourceDescriptor:
        """ChatModel which focus on text generate, and reuse ChatModelConnection."""
        return ResourceDescriptor(
            clazz=ResourceName.ChatModel.JAVA_WRAPPER_SETUP,
            java_clazz=ResourceName.ChatModel.Java.OLLAMA_SETUP,
            connection="ollama_connection_java",
            model=os.environ.get("OLLAMA_CHAT_MODEL", "qwen3:1.7b"),
            prompt="from_text_prompt",
            extract_reasoning=True,
        )

    @tool
    @staticmethod
    def add(a: int, b: int) -> int:
        """Calculate the sum of a and b.

        Parameters
        ----------
        a : int
            The first operand
        b : int
            The second operand

        Returns:
        -------
        int:
            The sum of a and b
        """
        return a + b

    @action(InputEvent)
    @staticmethod
    def process_input(event: InputEvent, ctx: RunnerContext) -> None:
        """User defined action for processing input.

        In this action, we will send ChatRequestEvent to trigger built-in actions.
        """
        input_text = event.input.lower()
        model_name = (
            "math_chat_model"
            if ("calculate" in input_text or "sum" in input_text)
            else "creative_chat_model"
        )
        ctx.send_event(
            ChatRequestEvent(
                model=model_name,
                messages=[ChatMessage(role=MessageRole.USER, content=event.input)],
            )
        )

    @action(ChatResponseEvent)
    @staticmethod
    def process_chat_response(event: ChatResponseEvent, ctx: RunnerContext) -> None:
        """User defined action for processing chat model response."""
        input = event.response
        if event.response and input.content:
            ctx.send_event(OutputEvent(output=input.content))



################################################################################
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
# limitations under the License.
#################################################################################

from typing import Any, Dict, List

from typing_extensions import override

from flink_agents.api.vector_stores.java_vector_store import (
    JavaCollectionManageableVectorStore,
)
from flink_agents.api.vector_stores.vector_store import (
    Collection,
    Document,
    VectorStoreQuery,
    VectorStoreQueryResult,
    _maybe_cast_to_list,
)
from flink_agents.runtime.python_java_utils import (
    from_java_collection,
    from_java_document,
    from_java_vector_store_query_result,
)


class JavaVectorStoreImpl(JavaCollectionManageableVectorStore):
    """Java-based implementation of EmbeddingModelSetup that wraps a Java embedding
    model object.
    This class serves as a bridge between Python and Java embedding model environments,
    but unlike JavaEmbeddingModelConnection, it does not provide direct embedding
    functionality in Python.
    """
    _j_resource: Any
    _j_resource_adapter: Any

    def __init__(self, j_resource: Any, j_resource_adapter: Any, **kwargs: Any) -> None:
        """Creates a new JavaEmbeddingModelSetup.

        Args:
            j_resource: The Java resource object
            j_resource_adapter: The Java resource adapter for method invocation
            **kwargs: Additional keyword arguments
        """
        # embedding_model are required parameters for BaseVectorStore
        embedding_model = kwargs.pop("embedding_model", "")
        super().__init__(embedding_model = embedding_model, **kwargs)

        self._j_resource=j_resource
        self._j_resource_adapter=j_resource_adapter

    @override
    @property
    def store_kwargs(self) -> Dict[str, Any]:
        return {}

    @override
    def add(
            self,
            documents: Document | List[Document],
            collection_name: str | None = None,
            **kwargs: Any,
    ) -> List[str]:

        documents = _maybe_cast_to_list(documents)
        j_documents = [
            self._j_resource_adapter.fromPythonDocument(document)
            for document in documents
        ]

        return self._j_resource.add(j_documents, collection_name, kwargs)

    @override
    def query(self, query: VectorStoreQuery) -> VectorStoreQueryResult:
        j_query = self._j_resource_adapter.fromPythonVectorStoreQuery(query)
        j_query_result = self._j_resource.query(j_query)
        return from_java_vector_store_query_result(j_query_result)

    @override
    def size(self, collection_name: str | None = None) -> int:
        return self._j_resource.size(collection_name)

    @override
    def get(
            self,
            ids: str | List[str] | None = None,
            collection_name: str | None = None,
            **kwargs: Any,
    ) -> List[Document]:
        ids = _maybe_cast_to_list(ids)
        j_documents = self._j_resource.get(ids, collection_name, kwargs)
        return [from_java_document(j_document) for j_document in j_documents]

    @override
    def delete(
        self,
        ids: str | List[str] | None = None,
        collection_name: str | None = None,
        **kwargs: Any,
    ) -> List[str]:
        ids = _maybe_cast_to_list(ids)
        return self._j_resource.delete(ids, collection_name, kwargs)

    @override
    def get_or_create_collection(
            self, name: str, metadata: Dict[str, Any] | None = None
    ) -> Collection:
        j_collection = self._j_resource.getOrCreateCollection(name, metadata)
        return from_java_collection(j_collection)

    @override
    def get_collection(self, name: str) -> Collection:
        j_collection = self._j_resource.getCollection(name)
        return from_java_collection(j_collection)

    @override
    def delete_collection(self, name: str) -> Collection:
        j_collection = self._j_resource.deleteCollection(name)
        return from_java_collection(j_collection)

    @override
    def _add_embedding(
        self,
        *,
        documents: List[Document],
        collection_name: str | None = None,
        **kwargs: Any,
    ) -> List[str]:
        """Private functions should never be called for the Resource Wrapper."""

    @override
    def _query_embedding(
        self, embedding: list[float], limit: int = 10, **kwargs: Any
    ) -> list[Document]:
        """Private functions should never be called for the Resource Wrapper."""
